/*============================================================================*
 * FILE:                v x w B o a r d S e t u p V M E . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2016 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools/1553-API Library:
 *             Low-level board setup component.
 *
 *             This module maps the board address and determines the setup
 *             function that should run based on card and carrier type.  This
 *             routine is for use with VxWorks systems running PowerPC or
 *             Motorola 68K and using the VME-1553 or the IP-1553 on a VME
 *             carrier board.
 *
 * API ENTRY POINTS: (only used by btdrv.c):
 *    vbtBoardAccessSetup adapter page access data pointers for VME boards.
 *
 *===========================================================================*/

/* $Revision:  8.14 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  01/07/2002   Initial release.
  12/07/2006   Update for O/S independent build
  11/05/2014   Fix error return
  12/05/2017   Updated for bt_iobase type change from CEI_ULONG[] to PCHAR[].
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "busapi.h"
#include "apiint.h"
#include "globals.h"
#include "btdrv.h"
#include "lowlevel.h"


/*===========================================================================*
 * LOCAL ENTRY POINT:      v b t B o a r d A c c e s s S e t u p
 *===========================================================================*
 *
 * FUNCTION:    Setup specified adapter for read/write access.
 *
 * DESCRIPTION: Initialize the page access data elements and map the board
 *              into the host address space..
 *
 *      It will return:
 *              0 if successful, non-zero upon error.
 *===========================================================================*/

BT_INT vbtBoardAccessSetup(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT phys_addr)
{
   BT_INT     status;
   char      *hostaddr;        // Mapped address of the board in host space.
   
   /*********************************************************************
   *  First verify that this is a board type we can handle.
   *********************************************************************/
   if (CurrentCardType[cardnum] != QVME1553)
      return BTD_ERR_BADBOARDTYPE;    // Unknown board type

   /*********************************************************************
   *  Now map the board address(es) and the I/O address.
   *********************************************************************/

   /* Get the local address of the VME memory space */
   status = vbtMapBoardAddress(phys_addr,0x2000000,
                    (void *)&bt_PageAddr[cardnum][0],&cardnum,CurrentMemMap[cardnum]);        /* Memory space. */
   if ( status != BTD_OK )
      return (status);
   hostaddr = bt_PageAddr[cardnum][0];

   /* Get the local address of the VME IO space */
   status = vbtMapIoAddress((CEI_ULONG)(bt_iobase[cardnum]), (void *)&bt_PageAddr[cardnum][1]);    /* IO space. */
   if ( status != BTD_OK )
      return (status);
      
   bt_iobase[cardnum] = bt_PageAddr[cardnum][1];

   boardBaseAddress[cardnum] = hostaddr;

   /*********************************************************************
   *  Setup the access parameters for the VME-1553.
   *********************************************************************/
   if (CurrentCardType[cardnum] == QVME1553 )
   {
      return vbtPageAccessSetupVME(cardnum, (unsigned)phys_addr, hostaddr);
   }
   else
      return BTD_NON_SUPPORT;
}
