/*============================================================================*
 * FILE:               V M E _ I N T _ S E T U P _ V M E . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2019 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *
 *===========================================================================*
 *
 * FUNCTION:   Set up the interrupt for x86  VxWorks using PCI-1553 interface.  
 *             Allows user to add their function called on the interrupt.
 *
 *
 * API ENTRY POINTS: none
 *                  
 *
 * MAJOR INTERNAL ROUTINES:
 *    irq_setup            Connects the interrupt to the ISR.
 *    intProcess           Interrupt Service Routine.
 *
 *===========================================================================*/

/* $Revision:  8.28 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  01/18/2005    Initial Version.
  03/20/2009    Change the interrupt level and vectors.
  11/05/2014    Update for V6 compatibility
  12/26/2019    Fixed vbtSetPolling by adding CEI_THREAD_DESTROY. bch
 */

/*---------------------------------------------------------------------------*
 *                    INCLUDES, DEFINES and GLOBALS
 *---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <intLib.h>
#include <logLib.h>
#include <semLib.h>
#include <sigLib.h>
#include <sysLib.h>
//#include <taskLib.h>
#include <errno.h>
#include "busapi.h"
#include "apiint.h"
#include "btdrv.h"
#include "globals.h"
//#define _BT_GLOBALS_  /* This module allocates the memory for the driver */
//#define SYS_VME_INT_CONNECT 
#define GE_VME
#ifdef GE_VME
STATUS gefVmeIntConnect (
                        VOIDFUNCPTR pFunc,
                        int intLevel,  /* GEF_VME_INT_LEVEL */
                        int intVector,
                        int arg  /* _Vx_usr_arg_t arg */
                        );
#endif
clock_t clock_id;
//timer_t iTimer;
//struct itimerspec tValue;   /* time to be set */
//struct sigevent tevp;
int timerId = 0;
static BT_U32BIT bmRecordTime[MAX_BTA] = {0,};
//static int hIntEvent[MAX_BTA];          // Interrupt signal event, if it exists.

#define CLOCK_RATE 100 // This gives 10 millisecond clock ticks increase for 
                       // faster clock rate. 

SEM_ID intSem[MAX_BTA];
int task_id[MAX_BTA];

int irq_lvl[] = {0x5,0x5,0x5,0x5,   //default IRQ Level for up to 4 QVME/VME-1553
                 0x5,0x5,0x5,0x5,
                 0x5,0x5,0x5,0x5, 
                 0x5,0x5,0x5,0x5};

int int_vector[] = {0x80,0x81,0x82,0x83,  //Default interrupt vector for 16 channels
	                0x84,0x85,0x86,0x87,
	                0x88,0x89,0x90,0x91,
	                0x92,0x93,0x94,0x95};

int irq_lvl_use_cnt[] = {0,0,0,0,0,0,0};
int vme_channel_offset[] = {CHAN1_VME,CHAN2_VME,CHAN3_VME,CHAN4_VME};
char intproc[16];

static unsigned int_count[4];
CEI_THREAD polling_timer_thread;

void (*intFptr[MAX_BTA])(int);
void user_int_process(int);
static void int_notice(int,int,int,int,int,int,int,int,int,int);
void intProcess(int);
void* polling_timer_func(void* val);

#define POLLING_TIMER_THREAD_PRIORITY 4

/*===========================================================================*
 * ENTRY POINT:             i r q _ s e t u p
 *===========================================================================*
 *
 * FUNCTION:    Setup specified adapter for interrupt processing.
 *
 * DESCRIPTION: Setup the hardware interrupt if hardware interrupts are
 *              enabled, and setup the polling interrupt if not setup already.
 *
 *      It will return:
 *              Nothing.
 *===========================================================================*/
int vbtInterruptSetup(BT_UINT cardnum,  BT_INT hw_int_enable, BT_INT api_dev)
{
   /*******************************************************************
   *  Local variables
   *******************************************************************/
   volatile  short *vme_config_addr; // word poinyer to VME/VXI config registers
   short cdata;
   BT_UINT status;

   /*******************************************************************
   *  Setup the hardware interrupt function if interrupts enabled,
   *   but only if HW interrupt is defined and if this is not the
   *   software version of the driver.
   *******************************************************************/
   intFptr[cardnum] = (void *)vbtNotify[cardnum];
   status = API_SUCCESS;

   if(hw_int_enable == API_SW_INTERRUPT) //Polling mode no H/W interrupts
   {
      if(timerId==0)
      {
         status = vbtSetPolling(api_polling_interval,TIMER_START);
         timerId = 1;
      }
      else
      {
         timerId++;
      }
      return status;
   }

   // Clear interrupt by writing a zero to WRITE_INTERRUPT_BIT
   if(board_is_v5_uca[cardnum])
      vbtSetHWRegister(cardnum, HWREG_WRITE_INTERRUPT, 0);
   else
      api_clearhwcbits(cardnum,CR1_HOST_INTERP);
      
   if(sysIntDisable(irq_lvl[cardnum]) == ERROR) // disable VME level
      return 1000;//ERROR;
      
#ifdef SYS_VME_INT_CONNECT
   if( sysVmeIntConnect(INUM_TO_IVEC(int_vector[cardnum]), intProcess, cardnum) == ERROR) // connect ISR to VME Vector UNIV_INT_VEC
#elif defined (GE_VME)
   if(gefVmeIntConnect(intProcess,irq_lvl[cardnum],int_vector[cardnum],cardnum))
#else
   if(intConnect(INUM_TO_IVEC(int_vector[cardnum]), intProcess, cardnum) == ERROR) // connect ISR to VME Vector UNIV_INT_VEC
#endif
      return 2000;//ERROR;

   // Set up the interrupt vector on the VME-1553
   vme_config_addr = (short *)bt_iobase[cardnum];
   cdata = flipws(vme_config_addr[4]);
   cdata &=0xfff8;
   cdata = cdata | irq_lvl[cardnum] | 0x80;
   vme_config_addr[4] = flipws(cdata);
   vme_config_addr[7 + CurrentCardSlot[cardnum]] = flipws(int_vector[cardnum]);    /* setup interrupt vector for channel */
      
   if(sysIntEnable(irq_lvl[cardnum]) == ERROR) // enable VME level
      return 3000;//ERROR;

   irq_lvl_use_cnt[irq_lvl[cardnum]]++; 
   int_count[cardnum] = 0;  // clear interrupt count
      
   /* create semaphore */ 
   intSem[cardnum] = semBCreate (SEM_Q_FIFO, SEM_EMPTY);
   if(intSem[cardnum] == NULL)
      return BTD_SEM_CREATE;

   /* spawn task used for synchronization. */
   sprintf(intproc,"interrupt_%d_%d",api_dev,cardnum);
   task_id[cardnum] = taskSpawn (intproc, 64 + cardnum, VXW_TASK_OPTION, 20000, (FUNCPTR)int_notice,cardnum,0,0,0,0,0,0,0,0,0);
   if(task_id[cardnum] == ERROR)
      return BTD_TASK_CREATE;

   if(board_is_v5_uca[cardnum])
      api_writehwreg_or(cardnum, HWREG_CONTROL1, CR1_INT_ENABLE);
   else   
      api_sethwcbits(cardnum, CR1_INT_ENABLE);

   return 0;
}

BT_INT BusTools_GetIntCount(BT_UINT cardnum, BT_UINT *count)
{
   if (cardnum >= MAX_BTA)
      return API_BUSTOOLS_BADCARDNUM;

   *count = int_count[cardnum];

   return API_SUCCESS;
}

BT_INT BusTools_SetIntVector(BT_UINT cardnum, BT_UINT vector) //
{
   if (cardnum >= MAX_BTA)
      return API_BUSTOOLS_BADCARDNUM;

   if(vector == 0 || vector > 255)
      return API_BAD_PARAM;

   int_vector[cardnum] = vector;

   return API_SUCCESS;
}

int BusTools_SetIRQ_Lvl(BT_UINT cardnum, BT_UINT irq)  //
{
   if (bt_inited[cardnum] != 0)
      return API_BUSTOOLS_INITED;

   if (cardnum >= MAX_BTA)
      return API_BUSTOOLS_BADCARDNUM;
 
   if((irq == 0) || (irq > 7))
     return API_BAD_PARAM;

   irq_lvl[cardnum] = irq;

   return API_SUCCESS;
}

void intProcess(int cardnum)
{
   unsigned short control_data;
   short *HWR_Addr; 

   HWR_Addr = (short *)(bt_PageAddr[cardnum][3] + vme_channel_offset[CurrentCardSlot[cardnum]] + HW_REG_VME);          // HW Registers

   // read the 1553_control_register on channel 1 
   control_data = *HWR_Addr;

   if(control_data & 0x200)
   {
      // write 0 to the write_interrupt_bit     
      HWR_Addr[0xb] = 0x0;  // Clear the interrupt on Channel 1 
      semGive(intSem[cardnum]);
      int_count[cardnum]++;
   }
}

void int_notice(int cardnum,int a,int b,int c,int d,int e,int f,int g,int h,int j)
{
   while(1)
   {
      semTake (intSem[cardnum], WAIT_FOREVER); /* wait for event to occur */
      (intFptr[cardnum])(cardnum);
   }
}

void vbtInterruptClose(BT_UINT cardnum)
{
   int device;
   
   device = api_device[cardnum];

   if ( --timerId==0 )
   {
      CEI_THREAD_EXIT(0, (CEI_VOID*)polling_timer_thread);
//      timer_delete(iTimer);
      timerId = 0;                      // The timer has been destroyed.  
   }

   semDelete(intSem[cardnum]);
   if(task_id[cardnum] !=0)
      taskDelete(task_id[cardnum]);

   if(--irq_lvl_use_cnt[irq_lvl[cardnum]]==0)
   {
      sysIntDisable(irq_lvl[cardnum]);
   }
}

int delay_ms(int ms) 
{
   struct timespec req, rem;

   memset(&req, 0, sizeof(req));
   memset(&rem, 0, sizeof(rem));

   req.tv_nsec = (ms * 1000000);

   while(1)
   {
      if(nanosleep(&req, &rem) == 0)
         break;
      if(errno != EINTR)
      {
         //printf(" <DEBUG> error in delay_ms - status: %d\n", errno);
         return errno;
      }
      //printf(" <DEBUG> delay_ms interrupted: %ld, %ld\n", rem.tv_nsec, rem.tv_sec);
      req.tv_nsec = rem.tv_nsec;
      req.tv_sec = rem.tv_sec;
   }
   return 0;
}


/*===========================================================================*
 * ENTRY POINT:           i n t T i m e r C a l l b a c k
 *===========================================================================*
 *
 * FUNCTION:    Windows multimedia timer callback procedure.
 *              See Multimedia function timeSetEvent() for details.
 *
 * DESCRIPTION: Count up the time to the BM Recording event.  Call vbt Notify
 *              for each board which is open, so it can process interrupts.
 *
 *      It will return:
 *              Nothing.
 *===========================================================================*/
 void* polling_timer_func(void* val) 
 {
    BT_INT cardnum;
    static BT_INT bcnt=0;
    BT_UINT beat;

    while(1)
    {
       delay_ms(api_polling_interval);

       for(cardnum=0; cardnum < MAX_BTA; cardnum++)
       {
          if(bt_inited[cardnum] <= 0)
             continue;

          if(++bcnt > 10)
          {
             if(board_is_v5_uca[cardnum])
                beat = vbtGetRegister16(cardnum,HWREG_HEART_BEAT);
             else
                beat = vbtGetRegister[cardnum](cardnum,HWREG_HEART_BEATV6);
  
             if(beat == heartbeat[cardnum]) //
                channel_status[cardnum].wcs_pulse=1;
             heartbeat[cardnum]=beat;
             bcnt=0;
		  }   

          // count the time to next BM recording event
          bmRecordTime[cardnum] += api_polling_interval;
          vbtNotify[cardnum](cardnum, &bmRecordTime[cardnum]);
       }
    }
 }


/*===========================================================================*
 * ENTRY POINT:           v b t S e t P o l l i n g
 *===========================================================================*
 *
 * FUNCTION:    The function sets the polling interval for the application
 *
 * DESCRIPTION: The function stop the timer set the timing interval and
 *              restarts the timer
 *
 *      It will return: status
 *             
 *===========================================================================*/
BT_INT vbtSetPolling(BT_UINT polling, // (i) polling interval
                     BT_UINT tflag)   // (i) timer option 0 - start; 1 - restart
{
   int status;

   if(tflag)
   {   
      if(timerId)  // if timer is running stop it
      {
         api_polling_interval = polling;
         if ( api_polling_interval > MAX_POLLING )
           api_polling_interval = MAX_POLLING;
         if ( api_polling_interval < MIN_POLLING )
            api_polling_interval = MIN_POLLING;
	    
//         timer_delete(iTimer);
         CEI_THREAD_DESTROY(&polling_timer_thread);
         timerId = 0;  // The timer has been destroyed.
      }
   }

   if(timerId == 0)
   {
      status = CEI_THREAD_CREATE(&polling_timer_thread, POLLING_TIMER_THREAD_PRIORITY, polling_timer_func, NULL);
      if(status)
         return status;
   }

   return BTD_OK;
}

BT_INT BusTools_DisableInt(BT_UINT cardnum)
{
   if (cardnum >= MAX_BTA)
      return API_BUSTOOLS_BADCARDNUM;

   vbtInterruptClose(cardnum);

   return API_SUCCESS;
}
