/*============================================================================*
 * FILE:                  v x w B o a r d S e t u p . C
 *============================================================================*
 *
 *      COPYRIGHT (C) 1998-2019 BY ABACO SYSTEMS, INC. 
 *      ALL RIGHTS RESERVED.
 *
 *      THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND
 *      COPIED ONLY IN ACCORDANCE WITH THE TERMS OF SUCH LICENSE AND WITH
 *      THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY
 *      OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE
 *      AVAILABLE TO ANY OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF THE
 *      SOFTWARE IS HEREBY TRANSFERRED.
 *
 *      THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT
 *      NOTICE AND SHOULD NOT BE CONSTRUED AS A COMMITMENT BY ABACO SYSTEMS.
 *
 *===========================================================================*
 *
 * FUNCTION:   BusTools/1553-API Library:
 *             This module contains the VxWorks specific board initialization,
 *===========================================================================*/
/* $Revision:  8.28 Release $
   Date        Revision
  ----------   ---------------------------------------------------------------
  11/05/2014   Update for V6 compatibility.
  12/05/2017   Updated for bt_iobase type change from CEI_ULONG[] to PCHAR[].
  09/03/2019   Renamed devices from gefesDev to avioDev.
  12/26/2019   Fixed BusTools_ListDevices. bch  
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ioLib.h>
#include <errno.h>
#include "busapi.h"
#include "apiint.h"
#include "globals.h"
#include "btdrv.h"
#include "lowlevel.h"
#include "avioVxwDrv.h"


#ifdef DEBUG
#define dprint(a,b) printf(a,b) 
#else
#define dprint(a,b)
#endif

/*===========================================================================*
 * LOCAL ENTRY POINT:      v b t B o a r d A c c e s s S e t u p
 *===========================================================================*
 *
 * FUNCTION:    Setup specified adapter for read/write access.
 *
 * DESCRIPTION: Initialize the page access data elements and map the board
 *              into the host address space..
 *
 *      It will return:
 *              0 if successful, non-zero upon error.
 *===========================================================================*/

BT_INT vbtBoardAccessSetup(
   BT_UINT   cardnum,       // (i) card number (0 based)
   BT_U32BIT phys_addr)
{
   BT_INT status=0;
   char *hostaddr=NULL;  /* Mapped address of the board in host space. */

   dprint("vbtBoardAccessSetup\n", 0);
   
   /*********************************************************************
    *  Now map the board address(es) and the I/O address.
    *********************************************************************/
   if((status = vbtMapBoardAddresses(phys_addr,&bt_devmap[cardnum])) != BTD_OK)
     return status;

   if(bt_devmap[cardnum].memSections == 1  || CurrentCardType[cardnum]==R15XMC2) {
     hostaddr = (char *)bt_devmap[cardnum].memHostBase[0];
   }
   else if(bt_devmap[cardnum].memSections == 2) {
     if((bt_devmap[cardnum].memLengthBytes[0] != 0x80) && (bt_devmap[cardnum].memLengthBytes[0] != 0x1000))
       return BTD_ERR_NOACCESS;

     if(bt_devmap[cardnum].memLengthBytes[1] != 0x00800000)
       return BTD_ERR_NOACCESS;
           
     hostaddr = (char *)bt_devmap[cardnum].memHostBase[1];
     bt_iobase[cardnum] = bt_devmap[cardnum].memHostBase[0]; 
   }
   else if(bt_devmap[cardnum].memSections == 3) {
     if((bt_devmap[cardnum].memLengthBytes[0] != 0x200) && (bt_devmap[cardnum].memLengthBytes[0] != 0x1000))
       return BTD_ERR_NOACCESS;

     if(bt_devmap[cardnum].memLengthBytes[2] != 0x00800000)
       return BTD_ERR_NOACCESS;
           
     hostaddr = (char *)bt_devmap[cardnum].memHostBase[2];
     bt_iobase[cardnum] = bt_devmap[cardnum].memHostBase[0];
   }
   boardBaseAddress[cardnum] = hostaddr;

   /*********************************************************************
   *  Setup the access parameters for the QPMC/QPM-1553 or RPCIe1553
   *********************************************************************/
   if(CurrentCardType[cardnum] == QPM1553 || CurrentCardType[cardnum] == RPCIe1553) {
     return vbtPageAccessSetupQPMC(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the QPCX-1553.
   *********************************************************************/
   if(CurrentCardType[cardnum] == QPCX1553) {
     return vbtPageAccessSetupQPCX(cardnum, hostaddr); 
   }

   /*********************************************************************
   *  Setup the access parameters for the QCP-1553.
   *********************************************************************/
   if(CurrentCardType[cardnum] == QCP1553) {
     return vbtPageAccessSetupQCP(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the Q104-1553P.
   *********************************************************************/
   if(CurrentCardType[cardnum] == Q1041553P) {
     return vbtPageAccessSetupQ104(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the RXMC-1553.
   *********************************************************************/
   if(CurrentCardType[cardnum] == RXMC1553) {
     return vbtPageAccessSetupRXMC(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the RXMC-1553.
   *********************************************************************/
   if(CurrentCardType[cardnum] == R15PMC) {
     return vbtPageAccessSetupR15PMC(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the RXMC2-1553.
   *********************************************************************/
   if(CurrentCardType[cardnum] == R15XMC2) {
     return vbtPageAccessSetupRXMC2(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the R15-LPCIE.
   *********************************************************************/
   if(CurrentCardType[cardnum] == LPCIE1553) {
     return vbtPageAccessSetupLPCIE(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the MPCIE-1553.
   *********************************************************************/
   if(CurrentCardType[cardnum] == MPCIE1553) {
     return vbtPageAccessSetupMPCIE(cardnum, hostaddr);
   }
   /*********************************************************************
   *  Setup the access parameters for the RAR15-XMC-XT/IT/FIO.
   *********************************************************************/
   if(CurrentCardType[cardnum] == RAR15XMCXT) {
     return vbtPageAccessSetupRAR15XT(cardnum, hostaddr);
   }
   
   return BTD_NON_SUPPORT;  
}

#ifdef VXW_VXB_DRIVER
BT_UINT read_vxb_data(void) {
   struct vxbDriverControl ctrl;
   
   dprint("read_vxb_data\n", 0);
   
   if(Av1553Data == NULL) {
     Av1553Data = (AVIO_DEVICE_INFO_TYPE*)malloc(sizeof(AVIO_DEVICE_INFO_TYPE));
     if(Av1553Data == NULL)
       return API_MEM_ALLOC_ERR;

     ctrl.driverName = AVIO_DRV_NAME;
     ctrl.drvCtrlCmd = VXB_GET_DEV_MAP;
     ctrl.drvCtrlInfo = Av1553Data;
     /* this will fail if the API calls this before the driver is configured */
     if(vxbDevMethodRun(DEVMETHOD_CALL(driverControl), &ctrl) != OK) {
       free(Av1553Data);
       Av1553Data = NULL;
       return BTD_ERR_NOTSETUP;
     }
   }
   
   return API_SUCCESS;
}

BT_INT close_vxb_channel(BT_INT cardnum)
{
   dprint("close_vxb_channel\n", 0);
   
   if(--(Av1553Data->dev_use_count[api_device[cardnum]])== 0) {
     free (Av1553Data);
   }
   Av1553Data=NULL;
   
   return API_SUCCESS;
}

/*
void CEI_OS_SHUTDOWN(cardnum)
{
   close_vxb_channel(cardnum);	
}
*/
#endif // VXW_VXB_DRIVER

/*===========================================================================*
 *  ENTRY POINT:     B u s T o o l s _ A P I _ O p e n C h a n n e l 
 *===========================================================================*
 *
 * FUNCTION:    Open 1553 device channel for use.
 *
 * DESCRIPTION: This function assigns the next available cardnum and assigns it 
 *              to the channel.  Then it starts the initialization process for 
 *              that channel.
 *
 *      It will return:
 *              0 if successful, non-zero upon error.
 *===========================================================================*/

NOMANGLE BT_INT CCONV BusTools_API_OpenChannel(
  BT_UINT   *chnd,       // (o) card number (0 - 12) (device number in 32-bit land)
  BT_UINT   mode,        // (i) operational mode
  BT_INT    devid,       // (i) linear base address of board/carrier memory area or WinRT device number
  BT_UINT   channel)     // (i) channel 1 - 4.
{
   BT_UINT status=0, cardnum=0;
   BT_UINT card_id[4][4] = {{0 ,1, 2, 3 },
                            {4 ,5, 6, 7 },
                            {8, 9, 10,11},
                            {12,13,14,15}};


   dprint("BusTools_API_OpenChannel\n", 0);
   
   if(devid < 0 || devid >= MAX_BTA)
     return API_BAD_DEVICE_ID;

   cardnum = card_id[devid][channel];
   *chnd = cardnum;
      
#ifdef VXW_VXB_DRIVER   
   if((status = read_vxb_data()) != API_SUCCESS)
     return status;
   
   if(Av1553Data->device_present[devid] == 0xcafe0000 + devid)
     Av1553Data->dev_use_count[devid]++;
   else
     return BTD_ERR_BADBOARDTYPE;

   CurrentCardType[cardnum] = Av1553Data->dev_id[devid];
#endif
   
#ifdef VXW_DRIVER_OPTION
   {
      char device_name[80];
      int fd=0, val=0;

      sprintf(device_name,"/%s/%d", AVIO_DEV_IO_NAME, devid);
      fd = open(device_name, O_RDONLY, 0);
      if(fd == -1)
        return BTD_ERR_BADOPEN;

      ioctl(fd, PCI_GET_BOARD_TYPE, &val);
      CurrentCardType[cardnum] = (BT_UINT)val & 0xFFFF;

      close(fd);
   }
#endif
   
   CurrentPlatform[cardnum] = PLATFORM_PC;            // Platform is PC.
   CurrentMemMap[cardnum]   = CARRIER_MAP_DEFAULT;    // No carrier mapping
   CurrentCardSlot[cardnum] = channel;                // Specified channel
   CurrentCarrier[cardnum]  = NATIVE;                 // No Carrier.

   status = API_Init(cardnum, (BT_U32BIT)devid, (BT_UINT)channel, &mode);
   if(status)
   {
      bt_inuse[cardnum] = 0;
   }
   return status;
}

/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_FindDevice
*
*  FUNCTION
*       This function finds the instance of the device.  If found it returns
*       the device number otherwise -1.
*
*  RETURNS
*       Device number or -1 on fail.
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_FindDevice(
  BT_INT device_type,  // (i) card number (0 - 15)
  BT_INT instance)     // (i) device instance (1 - 16)
{
   BT_INT i, dev_cnt=0;

#ifdef VXW_DRIVER_OPTION
   BT_INT status=0;
   char devname[80];
   int fd=0;
   unsigned int board_type=0;

   for(i=0;i<MAX_BTA;i++) {
     sprintf(devname,"/%s/%d", AVIO_DEV_IO_NAME, i);
     fd = open(devname, O_RDONLY, 0);
     if(fd == -1) {
       dprint("BusTools_FindDevice: ERROR - failed to open the device failed %s\n", devname);
       continue;
     }
	
     status = ioctl(fd, PCI_GET_BOARD_TYPE, &board_type);
     close(fd);
     if(status != 0) {
       dprint("BusTools_FindDevice: ERROR - ioctl failed\n", 0);
       continue;
     }
      
     if(board_type == device_type) {
       dev_cnt++;
       if(dev_cnt == instance)
         break;
     }
   }
#elif defined(VXW_VXB_DRIVER)
   BT_UINT status=0;
   
   if((status = read_vxb_data()) != API_SUCCESS) {
     dprint("BusTools_FindDevice: ERROR - read_vxb_data failed %d\n", status);
     return -1;
   }
   
   for(i=0; i<Av1553Data->number_of_devices; i++) {
     if(device_type == Av1553Data->dev_id[i]) {
       dev_cnt++;
       if(dev_cnt == instance)
         break;
     }
   }
#endif
   
   if(i == MAX_BTA)
     return -1;
   else
     return i; 
} 


/****************************************************************************
*
*  PROCEDURE NAME -   BusTools_ListDevices
*
*  FUNCTION
*       This function checks the windows registry for all installed Condor
*       Engineering boards and returns a list of those devices.
*
*  RETURNS
*       API_SUCCESS
*       BTD_ERR_BADADDRMAP
*
****************************************************************************/
NOMANGLE BT_INT CCONV BusTools_ListDevices(DeviceList *dlist)     // (i) channel 1 - 4.
{
#if defined(VXW_VXB_DRIVER)
   BT_UINT status=0, product_index=0;
   
   if((status = read_vxb_data()) != API_SUCCESS)
      return status;

   dlist->num_devices = Av1553Data->number_of_devices;
 
   for(product_index=0;product_index<dlist->num_devices;product_index++)
   {
      switch(Av1553Data->dev_id[product_index])
#elif defined(VXW_DRIVER_OPTION)
   BT_INT product_index,fd=0;
   unsigned int board_type=0;
   char devname[80];
      
   dlist->num_devices=0; 
   for(product_index=0; product_index<MAX_BTA; product_index++) {
     sprintf(devname,"/%s/%d", AVIO_DEV_IO_NAME, product_index);
     fd = open(devname, O_RDONLY,0);
     if(fd == -1)
       continue;

     ioctl(fd, PCI_GET_BOARD_TYPE, &board_type);
     close(fd);

     switch(board_type)
#endif      
     {               
       case QPM1553:
         strcpy((dlist->name[product_index]),"QPM(C)-1553");
         dlist->device_name[product_index] = QPMC1553;          // Card type QPMC-1553.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break;
       case QPCI1553:
         strcpy(dlist->name[product_index],"QPCI-1553"); 
         dlist->device_name[product_index] = QPCI1553;          // Card type QPCI-1553.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break;
       case QPCX1553:
         strcpy(dlist->name[product_index],"QPCX-1553"); 
         dlist->device_name[product_index] = QPCX1553;          // Card type QPCX-1553.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break;
       case RXMC1553:
         strcpy(dlist->name[product_index],"RXMC-1553"); 
         dlist->device_name[product_index] = RXMC1553;          // Card type RXMC-1553.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break;
       case R15PMC:
         strcpy(dlist->name[product_index],"R5-PMC"); 
         dlist->device_name[product_index] = R15PMC;            // Card type R15-PMC.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break;
       case RPCIe1553:
         strcpy(dlist->name[product_index],"RPCIe-1553"); 
         dlist->device_name[product_index] = RPCIe1553;          // Card type RPCIe-1553.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break;
       case QCP1553:
         strcpy(dlist->name[product_index],"QCP-1553"); 
         dlist->device_name[product_index] = QCP1553;          // Card type QCP-1553.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break;
       case Q1041553P:
         strcpy((dlist->name[product_index]),"Q104-1553P");
         dlist->device_name[product_index] = Q1041553P;          // Card type Q104-1553P.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break; 
       case RAR15XMCXT:
          strcpy(dlist->name[product_index],"RAR15-XMC-XT"); 
          dlist->device_name[product_index] = RAR15XMCXT;          // Card type RAR15-XMC-XT.
          dlist->device_num[product_index] = product_index;   
          break;    
       case MPCIE1553:
         strcpy((dlist->name[product_index]),"R15-MPCIE");
         dlist->device_name[product_index] = MPCIE1553;          // Card type Q104-1553P.
         dlist->device_num[product_index] = product_index;
//         dlist->num_devices++;
         break; 
       default:
         strcpy((dlist->name[product_index]),"-");
         dlist->device_name[product_index] = 0;   
         dlist->device_num[product_index] = -1;
     }
   } 
   
   return API_SUCCESS;              
}

#ifndef NO_HOST_TIME
/****************************************************************************
*
*  PROCEDURE - get_48BitHostTimeTag()
*
*  FUNCTION
*     This routine reads the host clock and returns an API-formatted initial
*     time tag value as a 64-bit integer in microseconds.  We wait for the
*     clock to change to make sure we get as accurate a time value as possible.
*
****************************************************************************/
void get_48BitHostTimeTag(BT_INT mode, BT1553_TIME *host_time)
{
   union bigtime
   {
	  CEI_UINT64 rtn_time;
	  BT1553_TIME rtn_val;
   }bt;
	

   time_t   now;             // Current time, modify to get Time Reference Point.
   struct   tm start;        // Formatted "now" current time, modified for TRP.
   time_t   then=0;            // Time Reference Point, Midnight or Jan 1.
   int status;
   struct timespec ts;        /* Current time, use to get milliseconds */

   unsigned last_milli;      // This is the time we use to wait for time to change.
   union{
		 CEI_UINT64 diff;            // This is the difference time we return in microseconds,
		 BT_U32BIT tt[2];
		}utime;                          //  either time since Midnight or time since Jan 1.
   switch (mode)
   {
   case API_TTI_DAY:  // Initial time relative to midnight
	  time(&now);               // Time in seconds since midnight Jan 1, 1970 GMT
	  start = *localtime(&now); // Local time in seconds, min, hour, day, month
	  start.tm_sec  = 0;        // Local
	  start.tm_min  = 0;        //       time
	  start.tm_hour = 0;        //            at midnight
	  then = mktime(&start);    // local midnight to time in seconds since Jan 1, 1970
	  break;

   case API_TTI_IRIG: // Initial time relative to Jan 1, this year
	  time(&now);               // Time in seconds since midnight Jan 1, 1970 GMT
	  start = *localtime(&now); // Local
	  start.tm_sec  = 0;        //   time
	  start.tm_min  = 0;        //     at
	  start.tm_hour = 0;        //       Jan 1
	  start.tm_mday = 0;        //         midnight (This is day 1)
	  start.tm_mon  = 0;        //           this
	  start.tm_yday = 0;        //             year
	  then = mktime(&start);    // local Jan 1 to time in seconds since Jan 1, 1970
								// We do IRIG time, which makes Jan 1 "day 1" */
	  break;
   }

   status = clock_gettime(CLOCK_REALTIME, &ts);
   last_milli = (unsigned long)ts.tv_nsec/1000000.;  /*  save millisecond */
   while (1)                 // Wait for milliseconds to chnage
   {
	  clock_gettime(CLOCK_REALTIME, &ts);
	  if ((unsigned long)(ts.tv_nsec/1000000.) != last_milli)

	  break;              // Break at exact time tick change
   }

   utime.tt[1] = ts.tv_sec - then;     /* We only want the time since Midnight or Jan  */
   utime.tt[0] = 0;
   utime.diff *= (CEI_UINT64)1000000;
   utime.diff += (ts.tv_nsec/1000); /*  plus number of Milliseconds converted to microseconds */
#ifdef NON_INTEL_WORD_ORDER
   {   
	  BT_U32BIT save;
      
	  save = utime.tt[1];
	  utime.tt[1] = flip(&utime.tt[0]);
	  utime.tt[0] = save;
   }
#endif //NON_INTEL_WORD_ORDER

   bt.rtn_time = utime.diff;        /* Pass it back to the caller */
   *host_time = bt.rtn_val; 
   return;
}
#endif //NO_HOST_TIME

#ifndef NO_HOST_TIME
/****************************************************************************
*
*  PROCEDURE - get_48BitHostTimeTag()
*
*  FUNCTION
*     This routine reads the host clock and returns an API-formatted initial
*     time tag value as a 64-bit integer in microseconds.  We wait for the
*     clock to change to make sure we get as accurate a time value as possible.
*
****************************************************************************/
void get_64BitHostTimeTag(BT_INT mode, BT1553_TIME *host_time)
{
   union bigtime
   {
	  CEI_UINT64 rtn_time;
	  BT1553_TIME rtn_val;
   }bt;
	

   time_t   now;             // Current time, modify to get Time Reference Point.
   struct   tm start;        // Formatted "now" current time, modified for TRP.
   time_t   then=0;            // Time Reference Point, Midnight or Jan 1.
   int status;
   struct timespec ts;        /* Current time, use to get milliseconds */

   unsigned last_milli;      // This is the time we use to wait for time to change.
   union{
		 CEI_UINT64 diff;            // This is the difference time we return in microseconds,
		 BT_U32BIT tt[2];
		}utime;                          //  either time since Midnight or time since Jan 1.
   switch (mode)
   {
   case API_TTI_DAY:  // Initial time relative to midnight
	  time(&now);               // Time in seconds since midnight Jan 1, 1970 GMT
	  start = *localtime(&now); // Local time in seconds, min, hour, day, month
	  start.tm_sec  = 0;        // Local
	  start.tm_min  = 0;        //       time
	  start.tm_hour = 0;        //            at midnight
	  then = mktime(&start);    // local midnight to time in seconds since Jan 1, 1970
	  break;

   case API_TTI_IRIG: // Initial time relative to Jan 1, this year
	  time(&now);               // Time in seconds since midnight Jan 1, 1970 GMT
	  start = *localtime(&now); // Local
	  start.tm_sec  = 0;        //   time
	  start.tm_min  = 0;        //     at
	  start.tm_hour = 0;        //       Jan 1
	  start.tm_mday = 0;        //         midnight (This is day 1)
	  start.tm_mon  = 0;        //           this
	  start.tm_yday = 0;        //             year
	  then = mktime(&start);    // local Jan 1 to time in seconds since Jan 1, 1970
								// We do IRIG time, which makes Jan 1 "day 1" */
	  break;
   }

   status = clock_gettime(CLOCK_REALTIME, &ts);
   last_milli = (unsigned long)ts.tv_nsec/1000000.;  /*  save millisecond */
   while (1)                 // Wait for milliseconds to chnage
   {
	  clock_gettime(CLOCK_REALTIME, &ts);
	  if ((unsigned long)(ts.tv_nsec/1000000.) != last_milli)

	  break;              // Break at exact time tick change
   }

   utime.tt[1] = ts.tv_sec - then;     /* We only want the time since Midnight or Jan  */
   utime.tt[0] = 0;
   utime.diff *= (CEI_UINT64)1000;
   utime.diff += (ts.tv_nsec/1000); /*  plus number of Milliseconds converted to microseconds */
#ifdef NON_INTEL_WORD_ORDER
   {   
	  BT_U32BIT save;
      
	  save = utime.tt[1];
	  utime.tt[1] = flip(&utime.tt[0]);
	  utime.tt[0] = save;
   }
#endif //NON_INTEL_WORD_ORDER

   bt.rtn_time = utime.diff;        /* Pass it back to the caller */
   *host_time = bt.rtn_val; 
   return;
}
#endif //NO_HOST_TIME
