/*===================================================================================================================*
 * FILE:                                   A V I O   V X W   D R V . H
 *===================================================================================================================*
 *
 *   COPYRIGHT 2011 - 2019 ABACO SYSTEMS, INC.  ALL RIGHTS RESERVED.
 *  
 *   THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED ONLY IN ACCORDANCE WITH THE
 *   TERMS OF SUCH LICENSE AND WITH THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY 
 *   OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON.
 *   NO TITLE TO AND OWNERSHIP OF THE SOFTWARE IS HEREBY TRANSFERRED.
 *  
 *   THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND SHOULD NOT BE CONSTRUED 
 *   AS A COMMITMENT BY ABACO SYSTEMS, INC.
 *
 *===================================================================================================================*
 *
 * FUNCTION:   Avionics I/O Boards VxBus Driver Definitions
 *
 *===================================================================================================================*/
/*
 *   Date       By   Modifications
 * ----------  ----  ----------------------------------------------------------
 * 09/20/2011   BC   Initial Release of VxBus Driver for QPM-1553 and RXMC-1553 
 * 08/06/2012   BC   Adds support for RAR15-XMC-XT
 * 03/19/2013   BC   Update for 64-bit driver.
 * 11/13/2013   BW   Add support for new ARINC boards
 * 11/13/2013   BC   Add support for QCP-1553
 * 11/05/2014   BC   Fix GEIP_XMC2_DEVID
 * 07/23/2015   BW   General rewrite and cleanup.
 * 12/11/2015   BW   Added defines for global message queue names AVIO_COM_MQ_NAME & AVIO_MSG_Q_NAME
 *                   Modified conflicting constant names with 1553 API lowlevel.h, adding DRV_ to 1553 board names
 *                   Added structure member msg_q_com_id to the dev ctrl struct def geipAv1553Ctrl, AVIO_DEVICE_CTRL_TYPE
 *                   Added structure member msg_q_com_id[] to the dev info struct def geipAv1553Data, AVIO_DEVICE_INFO_TYPE
 * 05/12/2016   BW   Renaming all things to generic corp references:
 *                      GEIP_DRV_H -> AVIO_VXW_DRV_H
 *                      GEIP_AV_DEBUG -> AVIO_DEBUG
 *                      GEIP_*_DEVID -> AVIO_PCI_*_DEVID
 *                      GEIP_*_ID -> AVIO_PCI_*_DEVID
 *                      GEIP_AV_* -> AVIO_*
 *                      GEIP_* -> AVIO_*
 *                      GEIP_AV_DEV_NAME "/geipAvDrv"  -> AVIO_DEV_NAME "/avioVxbDrv" 
 *                      GEIP_AV_DRV_NAME "geipAvDrv"   -> AVIO_DRV_NAME "avioVxbDrv"  
 *                      geipAvCfg1553Devices -> avioCfg1553Devices
 *                      geipAvConfigDevices -> avioConfigDevices
 *                      geipAvDrvRegister -> avioDrvRegister
 * 08/31/2016   BW   Moved all include files not required for API builds into avioVxwDrv.c
 * 12/26/2019   BH   Removed path from device and message queue define.
 *
 *===================================================================================================================*/
#ifndef AVIO_VXW_DRV_H
#define AVIO_VXW_DRV_H

#include <stdio.h>
#include <msgQLib.h>
#include <driverControl.h>            /* for struct vxbDriverControl       */
#ifdef SPIN_LOCK_PROTECT
#include <spinLockLib.h> 
#endif 


#ifdef AVIO_DEBUG
#define dprint(a,b) printf(a,b)
#else
#define dprint(a,b)
#endif

/* The following definitions define the number of supported devices */ 
#define DRV_MAX_DEVICES    16
#define MAX_CHANNELS       4
/*#define MAX_BTA            DRV_MAX_DEVICES*/

/* The following definitions are UCA32 1553 API specific board types */ 
#define DRV_RXMC1553       0x0260
#define DRV_QPM1553        0x0110
#define DRV_QCP1553        0x0210
#define DRV_R15XMC2        0x0300
#define DRV_RAR15XMC       0x0340
#define DRV_RAR15XMCXT     0x0360
#define DRV_R15PMC         0x0380
#define DRV_R15_MPCIE      0x0400

/* The following definitions are Avionics specific PCI Device ID's */ 
#define AVIO_PCI_QPM_DEVID       0x1553
#define AVIO_PCI_QCP_DEVID       0x1555
#define AVIO_PCI_XMC_DEVID       0x1559
#define AVIO_PCI_XMC2_DEVID      0x155c
#define AVIO_PCI_RAR15_DEVID     0x1530
#define AVIO_PCI_RAR15XT_DEVID   0x1542
#define AVIO_PCI_R15PMC_DEVID    0x1544
#define AVIO_PCI_R15_MPCIE_DEVID 0x1545
#define AVIO_PCI_CEI830_DEVID    0x0830
#define AVIO_PCI_P708_DEVID      0x0708
#define AVIO_PCI_PSER_DEVID      0x1004
#define AVIO_PCI_PDIS_DEVID      0x1006
#define AVIO_PCI_RAR_CPCI_DEVID  0x0630
#define AVIO_PCI_R830RX_DEVID    0x0831
#define AVIO_PCI_R830X820_DEVID  0x0832
#define AVIO_PCI_RCEI830A_DEVID  0x830A
#define AVIO_PCI_RP708_DEVID     0x708A
#define AVIO_PCI_RARXMC_DEVID    0x100C
#define AVIO_PCI_RAR_MPCIE_DEVID 0x100D

/* The following definition is the Avionics specific PCI Manufacturer ID */ 
#define AVIO_PCI_VENDID         0x13c6
#define AVIO_PCI_1553_VENDID    AVIO_PCI_VENDID

/* The following definitions are Avionics specific combined PCI Device ID and Manufacturer ID */ 
#define AVIO_PMC_ID           0x155313c6
#define AVIO_QCP_ID           0x155513c6
#define AVIO_XMC_ID           0x155913c6
#define AVIO_R15PMC_ID        0x154413c6
#define AVIO_XMC2_ID          0x155c13c6
#define AVIO_RAR15_ID         0x153013c6
#define AVIO_RAR15XT_ID       0x154213c6
#define AVIO_PCI_R15PMC_ID    0x154413c6
#define AVIO_PCI_R15_MPCIE_ID 0x154513c6
#define AVIO_P708_ID          0x070813c6
#define AVIO_RP708_ID         0x708a13c6
#define AVIO_PSER_ID          0x100413c6
#define AVIO_PDIS_ID          0x100613c6
#define AVIO_CEI830_ID        0x083013c6
#define AVIO_RAR_CPCI_ID      0x063013c6
#define AVIO_R830RX_ID        0x083113c6
#define AVIO_R830X820_ID      0x083213c6
#define AVIO_RCEI830A_ID      0x830a13c6
#define AVIO_RARXMC_ID        0x100c13c6
#define AVIO_PCI_RAR_MPCIE_ID 0x100D13c6

/* Avionics VxBus Driver Device Naming Convention */
#define AVIO_DEV_NAME      "avioVxbDrv" 
#define AVIO_DRV_NAME      "avioVxbDrv"
#define AVIO_MSG_Q_NAME    "msg_q_"
#define AVIO_COM_MQ_NAME   "com_msg_q_"

/* Avionics VxBus Driver API-specific Definitions */

/* API_VXB_METHOD_ERR is a rando value used only by the API to indicate an error was encountered 
 * in the invocation of the Avionics IO VxBus "other class driver" driver control 
 */
#define API_VXB_METHOD_ERR 0xBA5D;       

/* Avionics VxBus Driver-specific General Utility Definitions */
#define VXB_GET_DEV_MAP    0x13c67       /* some rando value passed by the application/API to the Avionics IO VxBus driver to access a board */
#define AVIO_MUL_PROT_TYP1 17            /* UCA32 board type for an RAR15-XMC Type 1A (a product never released) */
#define AVIO_MUL_PROT_TYP2 18            /* UCA32 board type for an RAR15-XMC-XT/IT/FIO */
#define ARINC_CHANNEL      4             /* The "ARINC Channel Indicator" for ISR processing on any RAR15-XMC product, regardless of 1553 channel count */
#define ARINC_INT_ASSERTED 0x10          /* CEI-x30 Global Enable and P-SER Control Register Interrupt Asserted Bit, bit 4 */
#define UCA_INT_ASSERTED   0x200         /* UCA 1553 Control Register Interrupt Asserted Bit, bit 9 */
#define ARINC_CLEAR_INT    0x40000010    /* CEI-x30 ONLY Atomic Clear Global Enable Register Interrupt Asserted register write value, clears bit 4 */
#define UCA32_CLEAR_INT    0x40000200    /* UCA32 ONLY Atomic Clear 1553 Control Register Interrupt Asserted register write value, clears bit 9 */

#define UCA32_1553_CTL_REG 0x0000C016    /* UCA32 offset from the base host interface to Channel 1 1553 Control Register */
#define UCA32_1553_CH_OFST 0x00001000    /* UCA32 offset between Channel memory regions in the host interface */
#define MULT_PT_ARINC_OFST 0x00040000    /* Multi-protocol board offset from the base host interface to the ARINC host interface */
#define UCA16_1553_CTL_REG 0x00000800    /* UCA<16> offset from the base host interface Channel 1 1553 Control Register */
#define UCA16_1553_ACK_REG 0x0000000B    /* UCA<16> offset from the base host interface Channel 1 1553 Interrupt Acknowledge Register */
#define UCA16_1553_CH1_OFF 0x00200000    /* UCA<16> offset to the Channel 1 memory region in the host interface */
#define UCA16_1553_CH2_OFF 0x00400000    /* UCA<16> offset to the Channel 1 memory region in the host interface */
#define UCA16_1553_CH3_OFF 0x00600000    /* UCA<16> offset to the Channel 1 memory region in the host interface */

#define BAR_SIZE_8MB       0x00800000
#define BAR_SIZE_4MB       0x00400000
#define BAR_SIZE_16MB      0x01000000
#define BAR_SIZE_512KB     0x00080000

/* PowerPC Endian Byte Swapping Macros */
#ifdef VXW_PCI_PPC
 #define flipw(word) *(word)= \
    (short)(((*(word) >> 8) & 0x00ff) | \
            ((*(word) << 8) & 0xff00))
#else
 #define flipw(word)
#endif

#ifdef VXW_PCI_PPC
#define fliplong(longword)  *(longword)= (((*longword & 0x000000ff) << 24) | \
                                          ((*longword & 0x0000ff00) << 8)  | \
                                          ((*longword & 0xff0000)   >> 8)  | \
                                          ((*longword & 0xff000000) >> 24))
#else
#define fliplong(longword)
#endif

#ifdef VXW_PCI_PPC
#define fliplongs(longword)  (((longword & 0x000000ff) << 24) | \
                              ((longword & 0x0000ff00) << 8)  | \
                              ((longword & 0xff0000)   >> 8)  | \
                              ((longword & 0xff000000) >> 24))
#else
#define fliplongs(longword) longword
#endif

/*-------------------------------------------------------------------------------------------------------------------*
 * Avionics I/O VxBus Driver Private and Public Device Information Data Structure Types
 *-------------------------------------------------------------------------------------------------------------------*/

/* Avionics VxBus Driver Local Individual Device Information */
typedef struct  
{	
   char * name;                         /* Driver instance name */
   unsigned char irq;                   /* Interrupt Request Level for this device */
   unsigned long size;                  /* Host interface BAR size, assigned but not used */
   VOID * baseAddressBar0;              /* Device BAR0 pointer as acquired from the VxWorks VxBus system */
   VOID * baseAddressBar2;              /* Device BAR2 pointer as acquired from the VxWorks VxBus system, if present on device */
   unsigned long membase;               /* Integer Storage of Device BAR0 pointer as acquired from the VxWorks VxBus system */                      
   unsigned long membaseBar2;           /* Integer Storage of Device BAR2 pointer as acquired from the VxWorks VxBus system, if present on device */
   int bustype;                         /* PCI/PCIe = 2 */
   unsigned short CondorDeviceID;       /* PCI Device ID */
   unsigned dev_id;                     /* 1553-specific Board Type, i.e 0x110 for QPMC-1553 */
   unsigned device;                     /* Device Index assigned to each board detected in avioConfigDevices */
   unsigned dev_chan;                   /* Number of 1553 channels detected on a 1553 device */
   unsigned short hif;                  /* Lower 16 bits of the first location in the respective board's host interface */
   int uca32;                           /* Indication of 1553 board type, for f/w v6 boards uca32 = 1, for f/w v4/5 boards uca32 = 0 */
   int arinc;                           /* Indication that ARINC is present on a multi-protocol (or ARINC) device, used in ISR */
   UINT region_count;                   /* Indicates the number of memory BAR's present on the device, 2 for any PLX based device, 1 everywhere else */
   UINT mem_base_reg;                   /* Indicates which of the device's BAR contains the host interface, 0 for BAR0 or 2 for BAR2 */
   MSG_Q_ID msg_q_id;                   /* The message queue ID assigned to the respective 1553 device for app-level ISR notification */
   MSG_Q_ID msg_q_com_id;               /* The message queue ID assigned to the respective 1553 device for app-level ISR notification */
   UINT devcnt;                         /* A running total of the number of devices assigned to the Avionics IO VxBus driver */
   void * baseHandle[6];                /* The physical address of each detected device BAR, passed to vxbRegMap */
} geipAv1553Ctrl, AVIO_DEVICE_CTRL_TYPE;

/* Avionics VxBus Driver Application/API Device Information for all discovered devices */
typedef struct 
{	
   unsigned char irq[DRV_MAX_DEVICES];             /* Interrupt Request Level for each device */
   unsigned long size[DRV_MAX_DEVICES];            /* Host interface BAR size, assigned but not used */
   VOID * baseAddressBar0[DRV_MAX_DEVICES];        /* BAR0 pointer as acquired from the VxWorks VxBus system for all Avionics IO devices */
   VOID * baseAddressBar2[DRV_MAX_DEVICES];        /* BAR2 pointer as acquired from the VxWorks VxBus system for all Avionics IO devices, if present on device */
   unsigned long membase[DRV_MAX_DEVICES];         /* Integer Storage of Device BAR0 pointer as acquired from the VxWorks VxBus system */
   unsigned long membaseBar2[DRV_MAX_DEVICES];     /* Integer Storage of Device BAR2 pointer as acquired from the VxWorks VxBus system, if present on device */
   unsigned short CondorDeviceID[DRV_MAX_DEVICES]; /* PCI Device ID for all detected Avionics IO devices */
   UINT dev_id[DRV_MAX_DEVICES];                   /* 1553-specific Board Type, i.e 0x110 for QPMC-1553 */
   UINT device[DRV_MAX_DEVICES];                   /* Device Index assigned to each board detected in avioConfigDevices */
   UINT dev_chan[DRV_MAX_DEVICES];                 /* Number of 1553 channels detected on each 1553 device */
   UINT uca32[DRV_MAX_DEVICES];                    /* Indication of 1553 board type, for f/w v6 boards uca32 = 1, for f/w v4/5 boards uca32 = 0 */
   UINT arinc[DRV_MAX_DEVICES];                    /* Indication that ARINC is present on a multi-protocol (or ARINC) device, used in ISR */      
   unsigned short hif[DRV_MAX_DEVICES];            /* Lower 16 bits of the first location in the respective board's host interface */
   MSG_Q_ID msg_q_id[DRV_MAX_DEVICES];             /* The message queue ID assigned to the respective device for app-level ISR notification */
   MSG_Q_ID msg_q_com_id[DRV_MAX_DEVICES];         /* The message queue ID assigned to the respective device for app-level ISR notification */
   UINT device_present[DRV_MAX_DEVICES];           /* Lower 16 bits are redundant to the "device[]" structure member, but with the constant 0xCAFE value in the upper 16 bits */
   UINT dev_use_count[DRV_MAX_DEVICES];            /* NOT USED... can it be deleted? */
   UINT number_of_devices;                         /* NOT USED... incremented but never properly initialized, can it be deleted? */
} geipAv1553Data, AVIO_DEVICE_INFO_TYPE;

/* Pointer for use in the BT-1553-API? can it be deleted? */
AVIO_DEVICE_INFO_TYPE *Av1553Data;

#ifdef SPIN_LOCK_PROTECT
/* ISR locking mechanism */
spinlockIsr_t spinlock[DRV_MAX_DEVICES];
#endif 

/*-------------------------------------------------------------------------------------------------------------------*
 * Avionics I/O VxBus Driver Local Function Prototypes
 *-------------------------------------------------------------------------------------------------------------------*/
/* ISR for legacy 1553 boards running UCA f/w v4/5. */
void intProcess(VXB_DEVICE_ID pInst); 

/* ISR for Avionics IO 1553 and MultiProtocol boards (UCA32 f/w v6). */
void intProcess32(VXB_DEVICE_ID pInst);

/* ISR for PMC Commercial Avionics IO cards (x30 and P-SER) */
void intProcessCommercial(VXB_DEVICE_ID pInst);

/* ISR for PCI Express Commercial Avionics IO cards (RAR-XMC) */
void intProcessCommercialPcie(VXB_DEVICE_ID pInst);

/* Configures Avionics 1553 device information in the structure available to the application. */
int  avioCfg1553Devices(AVIO_DEVICE_CTRL_TYPE * pDevCtrl);

/* This routine configures Avionics generic device information in the structure available to the application,
 *  as well as handling registration of the device BAR memory region(s) to VxWorks.  */
void avioConfigDevices(VXB_DEVICE_ID pInst);

/* This routine is referenced in the Avionics IO VxBus configuration file 40geipAvDrv.cdf.  It is the  
 * Avionics IO main VxBus Driver registration routine, notifying VxWorks the driver is available and registering
 * the structure "avioDevRegistration" as the source of information required for Avionics IO devices.  */
void avioDrvRegister(void);

#endif /* AVIO_VXW_DRV_H */
