/*===================================================================================================================*
 * FILE:                                   A V I O   V X W   D R V . H
 *===================================================================================================================*
 *
 *   COPYRIGHT 2017 - 2019 ABACO SYSTEMS, INC.  ALL RIGHTS RESERVED.
 *  
 *   THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED ONLY IN ACCORDANCE WITH THE
 *   TERMS OF SUCH LICENSE AND WITH THE INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY 
 *   OTHER COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY OTHER PERSON.
 *   NO TITLE TO AND OWNERSHIP OF THE SOFTWARE IS HEREBY TRANSFERRED.
 *  
 *   THE INFORMATION IN THIS SOFTWARE IS SUBJECT TO CHANGE WITHOUT NOTICE AND SHOULD NOT BE CONSTRUED 
 *   AS A COMMITMENT BY ABACO SYSTEMS, INC.
 *
 *===================================================================================================================*
 *
 * FUNCTION:   Avionics I/O Boards VxBus Gen2 Driver Definitions
 *
 *===================================================================================================================*/
/*
 *   Date    By   Modifications
 * --------  ---  ---------------------------------------------------------------------------------------------------
 * 09/28/17  GK   Created to support the VxBus Gen 2 driver implementation
 *                   Replaced debug printing via printf with kprintf
 *                   Added VXB_RESOURCE * types pIntRes, pMemRes0 and pMemRes2 to AVIO_DEVICE_CTRL_TYPE structure
 * 11/06/17  BDW  Modified AVIO_DEVICE_CTRL_TYPE and AVIO_DEVICE_INFO_TYPE structure member irq type from unsigned
 *                   char to unsigned int.
 * 11/18/18  DMH  Modified AVIO_DEVICE_CTRL_TYPE with VXB_DEV_ID and SL_NODE.
 * 09/12/19  BCH  Modified to include AVIO_VXW_DRV build option and added debug macros. 
 *
 *===================================================================================================================*/
#ifndef AVIO_VXW_DRV_H
#define AVIO_VXW_DRV_H


#include <hwif/vxbus/vxbLib.h>
#include <hwif/util/vxbResourceLib.h>
#if defined (AVIO_VXW_DRV)
#include <msgQLib.h>
/*#define AVIO_DEBUG*/  /* define to enable debugging */
#endif /* AVIO_VXW_DRV */

/* The following definitions define the number of supported devices */ 
#define DRV_MAX_DEVICES    16
#define MAX_CHANNELS       4

/* The following definitions are UCA32 1553 API specific board types */ 
#define DRV_RXMC1553       0x0260
#define DRV_QPM1553        0x0110
#define DRV_QCP1553        0x0210
#define DRV_R15XMC2        0x0300
#define DRV_RAR15XMC       0x0340
#define DRV_RAR15XMCXT     0x0360
#define DRV_R15PMC         0x0380
#define DRV_R15_MPCIE      0x0400

/* The following definitions are Avionics specific PCI Device ID's */ 
#define AVIO_PCI_QPM_DEVID       0x1553
#define AVIO_PCI_QCP_DEVID       0x1555
#define AVIO_PCI_XMC_DEVID       0x1559
#define AVIO_PCI_XMC2_DEVID      0x155c
#define AVIO_PCI_RAR15_DEVID     0x1530
#define AVIO_PCI_RAR15XT_DEVID   0x1542
#define AVIO_PCI_R15PMC_DEVID    0x1544
#define AVIO_PCI_R15_MPCIE_DEVID 0x1545
#define AVIO_PCI_CEI830_DEVID    0x0830
#define AVIO_PCI_P708_DEVID      0x0708
#define AVIO_PCI_PSER_DEVID      0x1004
#define AVIO_PCI_PDIS_DEVID      0x1006
#define AVIO_PCI_RAR_CPCI_DEVID  0x0630
#define AVIO_PCI_R830RX_DEVID    0x0831
#define AVIO_PCI_R830X820_DEVID  0x0832
#define AVIO_PCI_RCEI830A_DEVID  0x830A
#define AVIO_PCI_RP708_DEVID     0x708A
#define AVIO_PCI_RARXMC_DEVID    0x100C
#define AVIO_PCI_RAR_MPCIE_DEVID 0x100D

/* The following definition is the Avionics specific PCI Manufacturer ID */ 
#define AVIO_PCI_VENDID         0x13c6
#define AVIO_PCI_1553_VENDID    AVIO_PCI_VENDID

/* The following definitions are Avionics specific combined PCI Device ID and Manufacturer ID */ 
#define AVIO_PMC_ID           0x155313c6
#define AVIO_QCP_ID           0x155513c6
#define AVIO_XMC_ID           0x155913c6
#define AVIO_R15PMC_ID        0x154413c6
#define AVIO_XMC2_ID          0x155c13c6
#define AVIO_RAR15_ID         0x153013c6
#define AVIO_RAR15XT_ID       0x154213c6
#define AVIO_PCI_R15PMC_ID    0x154413c6
#define AVIO_PCI_R15_MPCIE_ID 0x154513c6
#define AVIO_P708_ID          0x070813c6
#define AVIO_RP708_ID         0x708a13c6
#define AVIO_PSER_ID          0x100413c6
#define AVIO_PDIS_ID          0x100613c6
#define AVIO_CEI830_ID        0x083013c6
#define AVIO_RAR_CPCI_ID      0x063013c6
#define AVIO_R830RX_ID        0x083113c6
#define AVIO_R830X820_ID      0x083213c6
#define AVIO_RCEI830A_ID      0x830a13c6
#define AVIO_RARXMC_ID        0x100c13c6
#define AVIO_PCI_RAR_MPCIE_ID 0x100D13c6

/* Avionics VxBus Driver Device Naming Convention */
#define AVIO_DRV_NAME      "avioVxbDrv"
#define AVIO_DEV_NAME      "avioVxbDev"
#define AVIO_DEV_IO_NAME   "avioVxbIoDev" 
#define AVIO_MSG_Q_NAME    "avio_msg_q_"
#define AVIO_COM_MQ_NAME   "avio_com_msg_q_"
#define AVIO_MEM_NAME      "avioMem"

/* Avionics VxBus Driver-specific General Utility Definitions */
#define VXB_GET_DEV_MAP    0x13c67       /* some random value passed by the application/API to the Avionics IO VxBus driver to access a board */
#define AVIO_MUL_PROT_TYP1 17            /* UCA32 board type for an RAR15-XMC Type 1A (a product never released) */
#define AVIO_MUL_PROT_TYP2 18            /* UCA32 board type for an RAR15-XMC-XT/IT/FIO */
#define ARINC_CHANNEL      4             /* The "ARINC Channel Indicator" for ISR processing on any RAR15-XMC product, regardless of 1553 channel count */
#define ARINC_INT_ASSERTED 0x10          /* CEI-x30 Global Enable and P-SER Control Register Interrupt Asserted Bit, bit 4 */
#define UCA_INT_ASSERTED   0x200         /* UCA 1553 Control Register Interrupt Asserted Bit, bit 9 */
#define ARINC_CLEAR_INT    0x40000010    /* CEI-x30 ONLY Atomic Clear Global Enable Register Interrupt Asserted register write value, clears bit 4 */
#define UCA32_CLEAR_INT    0x40000200    /* UCA32 ONLY Atomic Clear 1553 Control Register Interrupt Asserted register write value, clears bit 9 */

#define UCA32_1553_CTL_REG 0x0000C016    /* UCA32 offset from the base host interface to Channel 1 1553 Control Register */
#define UCA32_1553_CH_OFST 0x00001000    /* UCA32 offset between Channel memory regions in the host interface */
#define MULT_PT_ARINC_OFST 0x00040000    /* Multi-protocol board offset from the base host interface to the ARINC host interface */
#define UCA16_1553_CTL_REG 0x00000800    /* UCA<16> offset from the base host interface Channel 1 1553 Control Register */
#define UCA16_1553_ACK_REG 0x0000000B    /* UCA<16> offset from the base host interface Channel 1 1553 Interrupt Acknowledge Register */
#define UCA16_1553_CH1_OFF 0x00200000    /* UCA<16> offset to the Channel 1 memory region in the host interface */
#define UCA16_1553_CH2_OFF 0x00400000    /* UCA<16> offset to the Channel 1 memory region in the host interface */
#define UCA16_1553_CH3_OFF 0x00600000    /* UCA<16> offset to the Channel 1 memory region in the host interface */

#define BAR_SIZE_8MB       0x00800000
#define BAR_SIZE_4MB       0x00400000
#define BAR_SIZE_16MB      0x01000000
#define BAR_SIZE_512KB     0x00080000

/* API_VXB_METHOD_ERR is a random value used only by the API to indicate an error was encountered 
 * in the invocation of the Avionics IO VxBus "other class driver" driver control 
 */
#define API_VXB_METHOD_ERR 0xBA5D;

/* Avionics VxBus Driver Application/API Device Information for all discovered devices */
typedef struct avioAv1553Data
{  
   unsigned int irq[DRV_MAX_DEVICES];              /* Interrupt Request Level for each device */
   unsigned long size[DRV_MAX_DEVICES];            /* Host interface BAR size, assigned but not used */
   VOID * baseAddressBar0[DRV_MAX_DEVICES];        /* BAR0 pointer as acquired from the VxWorks VxBus system for all Avionics IO devices */
   VOID * baseAddressBar2[DRV_MAX_DEVICES];        /* BAR2 pointer as acquired from the VxWorks VxBus system for all Avionics IO devices, if present on device */
   unsigned long membase[DRV_MAX_DEVICES];         /* Integer Storage of Device BAR0 pointer as acquired from the VxWorks VxBus system */
   unsigned long membaseBar2[DRV_MAX_DEVICES];     /* Integer Storage of Device BAR2 pointer as acquired from the VxWorks VxBus system, if present on device */
   unsigned short CondorDeviceID[DRV_MAX_DEVICES]; /* PCI Device ID for all detected Avionics IO devices */
   UINT dev_id[DRV_MAX_DEVICES];                   /* 1553-specific Board Type, i.e 0x110 for QPMC-1553 */
   UINT device[DRV_MAX_DEVICES];                   /* Device Index assigned to each board detected in avioConfigDevices */
   UINT dev_chan[DRV_MAX_DEVICES];                 /* Number of 1553 channels detected on each 1553 device */
   UINT uca32[DRV_MAX_DEVICES];                    /* Indication of 1553 board type, for f/w v6 boards uca32 = 1, for f/w v4/5 boards uca32 = 0 */
   UINT arinc[DRV_MAX_DEVICES];                    /* Indication that ARINC is present on a multi-protocol (or ARINC) device, used in ISR */      
   unsigned short hif[DRV_MAX_DEVICES];            /* Lower 16 bits of the first location in the respective board's host interface */
   MSG_Q_ID msg_q_id[DRV_MAX_DEVICES];             /* The message queue ID assigned to the respective device for app-level ISR notification */
   MSG_Q_ID msg_q_com_id[DRV_MAX_DEVICES];         /* The message queue ID assigned to the respective device for app-level ISR notification */
   UINT device_present[DRV_MAX_DEVICES];           /* Lower 16 bits are redundant to the "device[]" structure member, but with the constant 0xCAFE value in the upper 16 bits */
   UINT dev_use_count[DRV_MAX_DEVICES];            /* NOT USED... can it be deleted? */
   UINT number_of_devices;                         /* NOT USED... incremented but never properly initialized, can it be deleted? */
} AVIO_DEVICE_INFO_TYPE;

/* used by the driver, driver methods and the APIs */
struct vxbDriverControl
{
   char *  driverName;
   int     drvCtrlCmd;
   void *  drvCtrlInfo;
};
typedef struct vxbDriverControl	VXB_DRV_CONTROL;

#if defined (VXW_VXB_DRIVER)
/* to support VxBus Gen2 driver methods */
#include <hwif/methods/avioVxwDrvMethod.h> /* include after the declaration of VXB_DRV_CONTROL */

/* function declarations */
IMPORT STATUS avioVxbDrvUnlink (VXB_DRV_CONTROL * pDrvCntrl);
IMPORT STATUS driverControl (VXB_DRV_CONTROL * pDrvCntrl);
IMPORT STATUS vxbDevMethodRun (VXB_METHOD_ID method, void * pArg);

#define	DEVMETHOD_CALL	VXB_DEVMETHOD_CALL

/* local method/function table */
LOCAL VXB_DRV_METHOD avioVxbMethods[] =
{
   { VXB_DEVMETHOD_CALL (driverControl), driverControl },
   { VXB_DEVMETHOD_CALL (vxbDrvUnlink),  avioVxbDrvUnlink },
   VXB_DEVMETHOD_END
};
#endif /* VXW_VXB_DRIVER */

#if defined (VXW_DRIVER_OPTION)
/*  IOCTL codes used in the VxBus2 RTP and the legacy driver */
#define PCI_SET_REGION      0x13c61
#define PCI_GET_REGION_MEM  0x13c62
#define PCI_GET_REGION_SIZE 0x13c63
#define PCI_GET_INTERRUPT   0x13c64
#define PCI_GET_NUM_CHAN    0x13c65
#define PCI_GET_NUM_REGS    0x13c68
#define PCI_GET_BOARD_TYPE  0x13c69
#define PCI_GET_PCIREGION_BYTE   0x13c6a
#define PCI_GET_PCIREGION_WORD   0x13c6b
#define PCI_GET_PCIREGION_DWORD  0x13c6c
#define PCI_GET_HOST_INTRF  0x13c6d
#define PCI_GET_MSG_Q_ID    0x13c6f
#define PCI_GET_CSC         0x13c601
#define PCI_SYS_CLK_SET     0x13c602
#define PCI_FREE_ADDR       0x13c603
#define PCI_GET_DEV_MAP     0x13c604
#define PCI_GET_DEVICE_ID   0x13c605

#define PHYSICAL_MAP 0
#define VIRTUAL_MAP 1

#if defined (AVIO_VXW_DRV)
/* defines from the lowlevel.h used in the APIs */
#define MAX_MEMORY   6
#define MAX_PORTS    2
#define CEI_HANDLE   int

typedef struct _DEVMAP_T {
   int            busType;                    /* One of BUS_TYPE. */
   int            interruptNumber;            /* Interrupt number. */
   int            memSections;                /* Number of memory regions defined. */
   int            flagMapToHost[MAX_MEMORY];  /* Set to map region into host space. */
   char         * memHostBase[MAX_MEMORY];    /* Base address of region in host space. */
   unsigned long  memStartPhy[MAX_MEMORY];    /* Physical base address of region. */
   unsigned int   memLengthBytes[MAX_MEMORY]; /* Length of region in bytes. */
   int            portSections;               /* Number of I/O port regions. */
   unsigned long  portStart[MAX_PORTS];       /* I/O Address of first byte. */
   unsigned int   portLength[MAX_PORTS];      /* Number of bytes in region. */
   int            llDriverVersion;            /* Low Level driver version. */
   int            KernelDriverVersion;        /* Kernel driver version. */
   CEI_HANDLE     hKernelDriver;              /* Handle to the kernel driver. */
   unsigned int   VendorID;                   /* Vendor ID if PCI card */
   unsigned int   DeviceID;                   /* Device ID if PCI card */
   int            device;                     /* This the device */
   int            use_count;                  /* Number of user channels */
   int            use_channel_map;            /* */
   int            mapping;                    /* Mapping option 0 - physical 1 - virtual */
} DEVMAP_T, *PDEVMAP_T;
#endif /* AVIO_VXW_DRV */
#endif /* VXW_DRIVER_OPTION */

#if defined(AVIO_VXW_DRV)
/* PowerPC Endian Byte Swapping Macros */
#ifdef VXW_PCI_PPC
 #define flipw(word) *(word)= (short)(((*(word) >> 8) & 0x00ff) | \
                              ((*(word) << 8) & 0xff00))
                              
 #define fliplong(longword)  *(longword)= (((*longword & 0x000000ff) << 24) | \
                                          ((*longword & 0x0000ff00) << 8)  | \
                                          ((*longword & 0xff0000)   >> 8)  | \
                                          ((*longword & 0xff000000) >> 24))

 #define fliplongs(longword)  (((longword & 0x000000ff) << 24) | \
                              ((longword & 0x0000ff00) << 8)  | \
                              ((longword & 0xff0000)   >> 8)  | \
                              ((longword & 0xff000000) >> 24))
#else
 #define flipw(word)
 #define fliplong(longword)
 #define fliplongs(longword) longword
#endif /* VXW_PCI_PPC */

#ifdef AVIO_DEBUG
#ifndef dprint
 #define dprint(a,b) kprintf(a,b)
#endif
#else
#ifndef dprint
 #define dprint(a,b)
#endif
#endif /* AVIO_DEBUG */

/*-------------------------------------------------------------------------------------------------------------------*
 * Avionics I/O VxBus Driver Private and Public Device Information Data Structure Types
 *-------------------------------------------------------------------------------------------------------------------*/
/* Avionics VxBus Driver Local Individual Device Information */
typedef struct avioAv1553Ctrl 
{
   SL_NODE avioNode;
   char * name;                         /* Driver instance name */
   unsigned int irq;                    /* Interrupt Request Level for this device */
   VXB_RESOURCE * pIntRes;              /* pointer to VxBus2 interrupt resource */
   VXB_RESOURCE * pMemRes0;             /* pointer to VxBus2 memory resource for BAR0 */
   VXB_RESOURCE * pMemRes2;             /* pointer to VxBus2 memory resource for BAR2 */
   unsigned long size;                  /* Host interface BAR size, assigned but not used */
   VOID * baseAddressBar0;              /* Device BAR0 pointer as acquired from the VxWorks VxBus system */
   VOID * baseAddressBar2;              /* Device BAR2 pointer as acquired from the VxWorks VxBus system, if present on device */
   unsigned long membase;               /* Integer Storage of Device BAR0 pointer as acquired from the VxWorks VxBus system */                      
   unsigned long membaseBar2;           /* Integer Storage of Device BAR2 pointer as acquired from the VxWorks VxBus system, if present on device */
   int bustype;                         /* PCI/PCIe = 2 */
   unsigned short CondorDeviceID;       /* PCI Device ID */
   unsigned short dev_id;               /* 1553-specific Board Type, i.e 0x110 for QPMC-1553 */
   unsigned int device;                 /* Device Index assigned to each board detected in avioConfigDevices */
   unsigned int dev_chan;               /* Number of 1553 channels detected on a 1553 device */
   unsigned short hif;                  /* Lower 16 bits of the first location in the respective board's host interface */
   int uca32;                           /* Indication of 1553 board type, for f/w v6 boards uca32 = 1, for f/w v4/5 boards uca32 = 0 */
   int arinc;                           /* Indication that ARINC is present on a multi-protocol (or ARINC) device, used in ISR */
   UINT region_count;                   /* Indicates the number of memory BAR's present on the device, 2 for any PLX based device, 1 everywhere else */
   UINT mem_base_reg;                   /* Indicates which of the device's BAR contains the host interface, 0 for BAR0 or 2 for BAR2 */
   MSG_Q_ID msg_q_id;                   /* The message queue ID assigned to the respective 1553 device for app-level ISR notification */
   MSG_Q_ID msg_q_com_id;               /* The message queue ID assigned to the respective 1553 device for app-level ISR notification */
   UINT devcnt;                         /* A running total of the number of devices assigned to the Avionics IO VxBus driver */
   void * baseHandle[6];                /* The physical address of each detected device BAR, passed to vxbRegMap */
   VXB_DEV_ID vxbDevId;                 /* VxBus device identifier */
} AVIO_DEVICE_CTRL_TYPE;

/*-------------------------------------------------------------------------------------------------------------------*
 * Avionics I/O VxBus Driver Local Function Prototypes
 *-------------------------------------------------------------------------------------------------------------------*/
/* ISR for legacy 1553 boards running UCA f/w v4/5. */
void intProcess(VXB_DEV_ID pInst); 

/* ISR for Avionics IO 1553 and MultiProtocol boards (UCA32 f/w v6). */
void intProcess32(VXB_DEV_ID pInst);

/* ISR for PMC Commercial Avionics IO cards (x30 and P-SER) */
void intProcessCommercial(VXB_DEV_ID pInst);

/* ISR for PCI Express Commercial Avionics IO cards (RAR-XMC) */
void intProcessCommercialPcie(VXB_DEV_ID pInst);

/* Configures Avionics 1553 device information in the structure available to the application. */
int  avioCfg1553Devices(AVIO_DEVICE_CTRL_TYPE * pDevCtrl);

/* This routine configures Avionics generic device information in the structure available to the application,
 *  as well as handling registration of the device BAR memory region(s) to VxWorks.  */
void avioConfigDevices(VXB_DEV_ID pInst);

#if defined (VXW_DRIVER_OPTION)
/* This routine is referenced in the Avionics IO VxBus configuration file 40geipAvDrv.cdf.  It is the  
 * Avionics IO main VxBus Driver registration routine, notifying VxWorks the driver is available and registering
 * the structure "avioDevRegistration" as the source of information required for Avionics IO devices.  */
STATUS avioIoDevRegister(VXB_DEV_ID vxbDevId);
#endif /* VXW_DRIVER_OPTION */

#else
AVIO_DEVICE_INFO_TYPE *Av1553Data;
#endif /* AVIO_VXW_DRV */
#endif /* AVIO_VXW_DRV_H */
